/****************************************************************************/
//    copyright 2012 -2016  Chris Rizzitello <sithlord48@gmail.com>         //
//                                                                          //
//    This file is part of FF7tk                                            //
//                                                                          //
//    FF7tk is free software: you can redistribute it and/or modify         //
//    it under the terms of the GNU General Public License as published by  //
//    the Free Software Foundation, either version 3 of the License, or     //
//    (at your option) any later version.                                   //
//                                                                          //
//   FF7tk is distributed in the hope that it will be useful,               //
//    but WITHOUT ANY WARRANTY; without even the implied warranty of        //
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          //
//    GNU General Public License for more details.                          //
/****************************************************************************/
#ifndef FF7CHAR_H
#define FF7CHAR_H

#include <QObject>
#include <QIcon>
#include <QStringList>

/*! \class FF7Char
*   \brief Data and Enums for Characters in Final Fantasy 7
*
*/

class FF7Char
{

public:
	enum FF7CharId{Cloud/**<0*/,Barret/**<1*/,Tifa/**<2*/,Aerith/**<3*/,Red/**<4*/,Yuffie/**<5*/,CaitSith/**<6*/,Vincent/**<7*/,Cid/**<8*/,YoungCloud/**<9*/,Sephiroth/**<10*/,Empty=0xFF/**<0xFF*/};
	enum FF7CharStatus
	{
		Sadness =0x10, /**<0x10*/
		Fury = 0x20,   /**<0x20*/
		BackRow = 0xFE,/**<0xFE*/
		FrontRow =0xFF /**<0xFF*/
	};
	enum CharSpecial
	{
		EmptyArmor=0xFF, /**<0xFF*/
		EmptyAccessory=0xFF/**<0xFF*/
	};
	explicit FF7Char();
	/*! \brief get id value for Character
	 * \param who persons id  (they are almost always the same in stock game)
	 * \return Chars[who].id
	 */
	quint8 id(int who);
	/*! \brief number of weapons for a Character
	 * \param who Characters id number
	 * \return Number of Weapons for the given character
	 */
	int numberOfWeapons(int who);

	/*! \brief find what item id is the first weapon for a Character
	 * \param who Characters id number
	 * \return Item Id of characters first weapon
	 */
	int weaponStartingId(int who);

	/*! \brief find what weapon number is the first weapon for a Character
	 * \param who Characters id number
	 * \return Weapon number of characters first weapon
	 */
	int weaponOffset(int who);

	/*! \brief default name for a character
	 * \param who Characters id number
     * \return Translated default name for character
	 */
	QString defaultName(int who);

	/*! \brief Menu icon for a character
	 * \param who Characters id number
	 * \return QImage of a characters menu icon
	 * \sa icon(),pixmap()
	 */
	QImage image(int who);

	/*! \brief Menu icon for a character
	 * \param who Characters id number
	 * \return QIcon of a characters menu icon
	 * \sa image(),pixmap()
	 */
	QIcon icon(int who);

	/*! \brief Menu icon for a character
	 * \param who Characters id number
	 * \return QPixmap of a characters menu icon
	 * \sa icon(),image()
	 */
	QPixmap pixmap(int who);

	/*! \brief Limit list for a character
	 * \param who Characters id number
	 * \return QStringList of Translated names for each limit break
	 */
	QStringList limits(int who);

	/*! \brief Exp needed to reach a level
	 * \param who Characters id number
	 * \param level Level you want to reach
	 * \return Total amount of Exp needed to reach level (ex. to reach level 99 you need 20000000 xp)
	 */
	quint32 totalExpForLevel(int who,int level);

	/*! \brief experance needed to level up
	 * \param who Characters id number
	 * \param level Level you are trying to reach
	 * \return Exp needed to reach the next level. (i.e from 9 to 10 you need 50 exp)
	 */
	quint32 tnlForLevel(int who,int level);

	/*! \brief how much to increase stat on level up
	 * \param who Characters id number
	 * \param stat Stat you want to gain a point in (stat: 0 str, 1 vit, 2 mag, 3 spi, 4 dex, 5 luck,6 hp,7mp)
	 * \param stat_amount current value for that stat
	 * \param current_lvl characters current level
	 * \param next_lvl level the character is becoming
	 * \return how many points to be put in stat.
	 */
	int statGain(int who,int stat,int stat_amount,int current_lvl, int next_lvl);
private:

	 /*! \brief characters grade in a stat
	 * \param who Characters id number
	 * \param stat Stat you want to get grade for (stat: 0 str, 1 vit, 2 mag, 3 spi, 4 dex, 5 luck,6 hp,7mp)
	 * \return characters grade in a stat
	 */
	quint8 stat_grade(int who,int stat);

	/*! \brief characters base mp for set of levels
	* \param who Characters id number
	* \param lvl_bracket bracket the level falls in (0: lvl 0-9 . 1: 10-19. etc..)
	* \return characters base mp for that bracket
	*/
	int mp_base(int who,int lvl_bracket);

	/*! \brief characters mp gradent for set of levels
	* \param who Characters id number
	* \param lvl_bracket bracket the level falls in (0: lvl 0-9 . 1: 10-19. etc..)
	* \return characters mp gradent for bracket
	*/
	quint8 mp_gradent (int who, int lvl_bracket);

	/*! \brief characters base hp for set of levels
	* \param who Characters id number
	* \param lvl_bracket bracket the level falls in (0: lvl 0-9 . 1: 10-19. etc..)
	* \return characters base hp for that bracket
	*/
	int hp_base(int who,int lvl_bracket);

	/*! \brief characters mp gradent for set of levels
	* \param who Characters id number
	* \param lvl_bracket bracket the level falls in (0: lvl 0-9 . 1: 10-19. etc..)
	* \return characters mp gradent for bracket
	*/
	quint8 hp_gradent (int who, int lvl_bracket);

	/*! \brief characters base luck for set of levels
	* \param who Characters id number
	* \param lvl_bracket bracket the level falls in (0: lvl 0-9 . 1: 10-19. etc..)
	* \return characters base luck for that bracket
	*/
	int luck_base(int who,int lvl_bracket);

	/*! \brief characters luck gradent for set of levels
	* \param who Characters id number
	* \param lvl_bracket bracket the level falls in (0: lvl 0-9 . 1: 10-19. etc..)
	* \return characters luck gradent for bracket
	*/
	quint8 luck_gradent (int who, int lvl_bracket);

	/*! \brief base stat for rank in a set of levels
	* \param rank Rank character is at start (grade)
	* \param lvl_bracket bracket the level falls in (0: lvl 0-9 . 1: 10-19. etc..)
	* \return base stat for that bracket
	*/
	quint8 stat_base (int rank,int lvl_bracket);

	/*! \brief characters mp gradent for set of levels
	* \param rank Rank character is at start (grade)
	* \param lvl_bracket bracket the level falls in (0: lvl 0-9 . 1: 10-19. etc..)
	* \return stat gradent for bracket
	*/
	quint8 stat_gradent (int rank,int lvl_bracket);
};

/*! \struct Character
 *  \brief Holds all data for a single Character
 *  \sa Chars[]
 */
struct Character {
	qint8 _id; /**< CharacterID */
	int _starting_weapon_id; /**< first item id for characters weapon*/
	int _num_weapons; /**< how many weapons the character has*/
	int _weapon_offset;/**< weapon offset of characters first weapon */
	QString _def_name; /**< default name for character*/
	QString _avatarString;/**< path it image resource*/
	QStringList _limits; /**< list of limit names */
	//Below for Leveling calcs.
	quint32 _charlvls[99];/**< holds exp needed for lvls */
	quint32 _chartnls[99];/**< starting exp needed to next level for each level*/
	quint8 _stat_grade[5];/**< how the character grades in each basic stat*/
	int _mp_base[8];/**< grade in mp gain for each level bracket*/
	quint8 _mp_gradent[8];/**< gradent for mp gain for each level bracket*/
	int _hp_base[8];/**< grade in hp gain for each level bracket*/
	quint8 _hp_gradent[8];/**< gradent for hp gain for each level bracket*/
	quint8 _luck_base[8];/**< luck base for each level bracket*/
	quint8 _luck_gradent[8];/**< gradent for luck for each level bracket*/
};

/*! \static Chars[]
 * Holds data for each character in Final Fantasy 7
 * \sa Character
 */
static const Character Chars[11]=
{
	{0,128,16,0,QT_TRANSLATE_NOOP("Char_Names","Cloud"),QString(":/characters/cloud"),
	QStringList()<<QT_TRANSLATE_NOOP("Char_Limits","Braver")<<QT_TRANSLATE_NOOP("Char_Limits","Cross-Slash")<<QT_TRANSLATE_NOOP("Char_Limits","Blade Beam")<<QT_TRANSLATE_NOOP("Char_Limits","Climhazzard")<<QT_TRANSLATE_NOOP("Char_Limits","Meteorain")<<QT_TRANSLATE_NOOP("Char_Limits","Finishing Touch")<<QT_TRANSLATE_NOOP("Char_Limits","Omnislash"),
	{static_cast<quint32>(0),static_cast<quint32>(6),static_cast<quint32>(33),static_cast<quint32>(94),static_cast<quint32>(202),static_cast<quint32>(372),static_cast<quint32>(616),static_cast<quint32>(949),static_cast<quint32>(1384),static_cast<quint32>(1934),static_cast<quint32>(2614),static_cast<quint32>(3588),static_cast<quint32>(4610),static_cast<quint32>(5809),static_cast<quint32>(7200),static_cast<quint32>(8797),static_cast<quint32>(10614),static_cast<quint32>(12665),static_cast<quint32>(14965),static_cast<quint32>(17528),static_cast<quint32>(20368),static_cast<quint32>(24161),static_cast<quint32>(27694),static_cast<quint32>(31555),static_cast<quint32>(35759),static_cast<quint32>(40321),static_cast<quint32>(45255),static_cast<quint32>(50576),static_cast<quint32>(56299),static_cast<quint32>(62438),static_cast<quint32>(69008),static_cast<quint32>(77066),static_cast<quint32>(84643),static_cast<quint32>(92701),static_cast<quint32>(101255),static_cast<quint32>(110320),static_cast<quint32>(119910),static_cast<quint32>(130040),static_cast<quint32>(140725),static_cast<quint32>(151980),static_cast<quint32>(163820),static_cast<quint32>(176259),static_cast<quint32>(189312),static_cast<quint32>(202994),static_cast<quint32>(217320),static_cast<quint32>(232305),static_cast<quint32>(247963),static_cast<quint32>(264309),static_cast<quint32>(281358),static_cast<quint32>(299125),static_cast<quint32>(317625),static_cast<quint32>(336872),static_cast<quint32>(356881),static_cast<quint32>(377667),static_cast<quint32>(399245),static_cast<quint32>(421630),static_cast<quint32>(444836),static_cast<quint32>(468878),static_cast<quint32>(493771),static_cast<quint32>(519530),static_cast<quint32>(546170),static_cast<quint32>(581467),static_cast<quint32>(610297),static_cast<quint32>(640064),static_cast<quint32>(670784),static_cast<quint32>(702471),static_cast<quint32>(735141),static_cast<quint32>(768808),static_cast<quint32>(803488),static_cast<quint32>(839195),static_cast<quint32>(875945),static_cast<quint32>(913752),static_cast<quint32>(952632),static_cast<quint32>(992599),static_cast<quint32>(1033669),static_cast<quint32>(1075856),static_cast<quint32>(1119176),static_cast<quint32>(1163643),static_cast<quint32>(1209273),static_cast<quint32>(1256080),static_cast<quint32>(1304080),static_cast<quint32>(1389359),static_cast<quint32>(1441133),static_cast<quint32>(1494178),static_cast<quint32>(1548509),static_cast<quint32>(1604141),static_cast<quint32>(1661090),static_cast<quint32>(1719371),static_cast<quint32>(1778999),static_cast<quint32>(1839990),static_cast<quint32>(1902360),static_cast<quint32>(1966123),static_cast<quint32>(2031295),static_cast<quint32>(2097892),static_cast<quint32>(2165929),static_cast<quint32>(2235421),static_cast<quint32>(2306384),static_cast<quint32>(2378833),static_cast<quint32>(2452783)},
	{static_cast<quint32>(0),static_cast<quint32>(6),static_cast<quint32>(27),static_cast<quint32>(61),static_cast<quint32>(108),static_cast<quint32>(170),static_cast<quint32>(244),static_cast<quint32>(333),static_cast<quint32>(435),static_cast<quint32>(550),static_cast<quint32>(680),static_cast<quint32>(974),static_cast<quint32>(1022),static_cast<quint32>(1199),static_cast<quint32>(1391),static_cast<quint32>(1597),static_cast<quint32>(1817),static_cast<quint32>(2051),static_cast<quint32>(2300),static_cast<quint32>(2563),static_cast<quint32>(2840),static_cast<quint32>(3793),static_cast<quint32>(3533),static_cast<quint32>(3861),static_cast<quint32>(4204),static_cast<quint32>(4562),static_cast<quint32>(4934),static_cast<quint32>(5321),static_cast<quint32>(5723),static_cast<quint32>(6139),static_cast<quint32>(6570),static_cast<quint32>(8058),static_cast<quint32>(7577),static_cast<quint32>(8058),static_cast<quint32>(8554),static_cast<quint32>(9065),static_cast<quint32>(9590),static_cast<quint32>(10130),static_cast<quint32>(10685),static_cast<quint32>(11255),static_cast<quint32>(11840),static_cast<quint32>(12439),static_cast<quint32>(13053),static_cast<quint32>(13682),static_cast<quint32>(14326),static_cast<quint32>(14985),static_cast<quint32>(15658),static_cast<quint32>(16346),static_cast<quint32>(17049),static_cast<quint32>(17767),static_cast<quint32>(18500),static_cast<quint32>(19247),static_cast<quint32>(20009),static_cast<quint32>(20786),static_cast<quint32>(21578),static_cast<quint32>(22385),static_cast<quint32>(23206),static_cast<quint32>(24042),static_cast<quint32>(24893),static_cast<quint32>(25759),static_cast<quint32>(26640),static_cast<quint32>(35297),static_cast<quint32>(28830),static_cast<quint32>(29767),static_cast<quint32>(30720),static_cast<quint32>(31687),static_cast<quint32>(32670),static_cast<quint32>(33667),static_cast<quint32>(34680),static_cast<quint32>(35707),static_cast<quint32>(36750),static_cast<quint32>(37807),static_cast<quint32>(38880),static_cast<quint32>(39967),static_cast<quint32>(41070),static_cast<quint32>(42187),static_cast<quint32>(43320),static_cast<quint32>(44467),static_cast<quint32>(45630),static_cast<quint32>(46807),static_cast<quint32>(48000),static_cast<quint32>(85279),static_cast<quint32>(51774),static_cast<quint32>(53045),static_cast<quint32>(54331),static_cast<quint32>(55632),static_cast<quint32>(56949),static_cast<quint32>(58281),static_cast<quint32>(59628),static_cast<quint32>(60991),static_cast<quint32>(62370),static_cast<quint32>(63763),static_cast<quint32>(65172),static_cast<quint32>(66597),static_cast<quint32>(68037),static_cast<quint32>(69492),static_cast<quint32>(70963),static_cast<quint32>(72449),static_cast<quint32>(73950)},
	 {1,6,3,4,26},{12,0,-26,-58,-102,-102,-4,180},{64,78,90,101,112,112,96,73},{200,-40,-640,-1440,-2280,-3080,-2040,-200},{19,42,72,100,121,137,120,98},{15,16,16,17,17,17,18,19},{20,10,10,8,8,8,7,6}},

	{1,160,16,32,QT_TRANSLATE_NOOP("Char_Names","Barret"),QString(":/characters/barret"),
	QStringList()<<QT_TRANSLATE_NOOP("Char_Limits","Big Shot")<<QT_TRANSLATE_NOOP("Char_Limits","Mindblow")<<QT_TRANSLATE_NOOP("Char_Limits","Grenade Bomb")<<QT_TRANSLATE_NOOP("Char_Limits","Hammerblow")<<QT_TRANSLATE_NOOP("Char_Limits","Satellite Beam")<<QT_TRANSLATE_NOOP("Char_Limits","Angermax")<<QT_TRANSLATE_NOOP("Char_Limits","Catastrophe"),
	 {static_cast<quint32>(0),static_cast<quint32>(7),static_cast<quint32>(35),static_cast<quint32>(98),static_cast<quint32>(210),static_cast<quint32>(385),static_cast<quint32>(637),static_cast<quint32>(980),static_cast<quint32>(1428),static_cast<quint32>(1995),static_cast<quint32>(2695),static_cast<quint32>(3689),static_cast<quint32>(4740),static_cast<quint32>(5973),static_cast<quint32>(7403),static_cast<quint32>(9045),static_cast<quint32>(10913),static_cast<quint32>(13022),static_cast<quint32>(15387),static_cast<quint32>(18022),static_cast<quint32>(20942),static_cast<quint32>(24827),static_cast<quint32>(28457),static_cast<quint32>(32424),static_cast<quint32>(36744),static_cast<quint32>(41431),static_cast<quint32>(46501),static_cast<quint32>(51968),static_cast<quint32>(57848),static_cast<quint32>(64155),static_cast<quint32>(70905),static_cast<quint32>(79149),static_cast<quint32>(86931),static_cast<quint32>(95207),static_cast<quint32>(103992),static_cast<quint32>(113302),static_cast<quint32>(123151),static_cast<quint32>(133555),static_cast<quint32>(144529),static_cast<quint32>(156088),static_cast<quint32>(168248),static_cast<quint32>(183403),static_cast<quint32>(196985),static_cast<quint32>(211222),static_cast<quint32>(226129),static_cast<quint32>(241721),static_cast<quint32>(258014),static_cast<quint32>(275023),static_cast<quint32>(292763),static_cast<quint32>(311250),static_cast<quint32>(330500),static_cast<quint32>(350527),static_cast<quint32>(371347),static_cast<quint32>(392976),static_cast<quint32>(415429),static_cast<quint32>(438721),static_cast<quint32>(462868),static_cast<quint32>(487885),static_cast<quint32>(513787),static_cast<quint32>(540590),static_cast<quint32>(568310),static_cast<quint32>(596961),static_cast<quint32>(626559),static_cast<quint32>(657120),static_cast<quint32>(688659),static_cast<quint32>(721191),static_cast<quint32>(754732),static_cast<quint32>(789297),static_cast<quint32>(824901),static_cast<quint32>(861560),static_cast<quint32>(899290),static_cast<quint32>(938105),static_cast<quint32>(978021),static_cast<quint32>(1019054),static_cast<quint32>(1061219),static_cast<quint32>(1104531),static_cast<quint32>(1149006),static_cast<quint32>(1194659),static_cast<quint32>(1241505),static_cast<quint32>(1289560),static_cast<quint32>(1338840),static_cast<quint32>(1389359),static_cast<quint32>(1441133),static_cast<quint32>(1494178),static_cast<quint32>(1548509),static_cast<quint32>(1604141),static_cast<quint32>(1661090),static_cast<quint32>(1719371),static_cast<quint32>(1778999),static_cast<quint32>(1839990),static_cast<quint32>(1902360),static_cast<quint32>(1966123),static_cast<quint32>(2031295),static_cast<quint32>(2097892),static_cast<quint32>(2165929),static_cast<quint32>(2235421),static_cast<quint32>(2306384),static_cast<quint32>(2378833),static_cast<quint32>(2452783)},
	 {static_cast<quint32>(0),static_cast<quint32>(7),static_cast<quint32>(28),static_cast<quint32>(63),static_cast<quint32>(112),static_cast<quint32>(175),static_cast<quint32>(252),static_cast<quint32>(343),static_cast<quint32>(448),static_cast<quint32>(567),static_cast<quint32>(700),static_cast<quint32>(994),static_cast<quint32>(1051),static_cast<quint32>(1233),static_cast<quint32>(1430),static_cast<quint32>(1642),static_cast<quint32>(1868),static_cast<quint32>(2109),static_cast<quint32>(2365),static_cast<quint32>(2635),static_cast<quint32>(2920),static_cast<quint32>(3885),static_cast<quint32>(3630),static_cast<quint32>(3967),static_cast<quint32>(4320),static_cast<quint32>(4687),static_cast<quint32>(5070),static_cast<quint32>(5467),static_cast<quint32>(5880),static_cast<quint32>(6307),static_cast<quint32>(6750),static_cast<quint32>(8244),static_cast<quint32>(7782),static_cast<quint32>(8276),static_cast<quint32>(8785),static_cast<quint32>(9310),static_cast<quint32>(9849),static_cast<quint32>(10404),static_cast<quint32>(10974),static_cast<quint32>(11559),static_cast<quint32>(12160),static_cast<quint32>(15155),static_cast<quint32>(13582),static_cast<quint32>(14237),static_cast<quint32>(14907),static_cast<quint32>(15592),static_cast<quint32>(16293),static_cast<quint32>(17009),static_cast<quint32>(17740),static_cast<quint32>(18487),static_cast<quint32>(19250),static_cast<quint32>(20027),static_cast<quint32>(20820),static_cast<quint32>(21629),static_cast<quint32>(22453),static_cast<quint32>(23292),static_cast<quint32>(24147),static_cast<quint32>(25017),static_cast<quint32>(25902),static_cast<quint32>(26803),static_cast<quint32>(27720),static_cast<quint32>(28651),static_cast<quint32>(29598),static_cast<quint32>(30561),static_cast<quint32>(31539),static_cast<quint32>(32532),static_cast<quint32>(33541),static_cast<quint32>(34565),static_cast<quint32>(35604),static_cast<quint32>(36659),static_cast<quint32>(37730),static_cast<quint32>(38815),static_cast<quint32>(39916),static_cast<quint32>(41033),static_cast<quint32>(42165),static_cast<quint32>(43312),static_cast<quint32>(44475),static_cast<quint32>(45653),static_cast<quint32>(46846),static_cast<quint32>(48055),static_cast<quint32>(49280),static_cast<quint32>(50519),static_cast<quint32>(51774),static_cast<quint32>(53045),static_cast<quint32>(54331),static_cast<quint32>(55632),static_cast<quint32>(56949),static_cast<quint32>(58281),static_cast<quint32>(59628),static_cast<quint32>(60991),static_cast<quint32>(62370),static_cast<quint32>(63763),static_cast<quint32>(65172),static_cast<quint32>(66597),static_cast<quint32>(68037),static_cast<quint32>(69492),static_cast<quint32>(70963),static_cast<quint32>(72449),static_cast<quint32>(73950)},
	 {5,2,18,14,29},{10,0,-20,-60,-108,-96,0,170},{57,67,77,90,102,100,84,63},{200,0,-760,-1840,-2840,-2840,-1160,600},{22,45,82,118,143,143,115,95},{14,15,15,15,16,17,18,20},{15,8,8,7,6,5,4,3}},

	{2,144,16,16,QT_TRANSLATE_NOOP("Char_Names","Tifa"),QString(":/characters/tifa"),
	QStringList()<<QT_TRANSLATE_NOOP("Char_Limits","Beat Rush")<<QT_TRANSLATE_NOOP("Char_Limits","Somersault")<<QT_TRANSLATE_NOOP("Char_Limits","Waterkick")<<QT_TRANSLATE_NOOP("Char_Limits","Meteodrive")<<QT_TRANSLATE_NOOP("Char_Limits","Dolphin Blow")<<QT_TRANSLATE_NOOP("Char_Limits","Meteor Strike")<<QT_TRANSLATE_NOOP("Char_Limits","Final Heaven"),
	{static_cast<quint32>(0),static_cast<quint32>(6),static_cast<quint32>(33),static_cast<quint32>(94),static_cast<quint32>(202),static_cast<quint32>(372),static_cast<quint32>(616),static_cast<quint32>(949),static_cast<quint32>(1384),static_cast<quint32>(1934),static_cast<quint32>(2614),static_cast<quint32>(3588),static_cast<quint32>(4610),static_cast<quint32>(5809),static_cast<quint32>(7200),static_cast<quint32>(8797),static_cast<quint32>(10614),static_cast<quint32>(12665),static_cast<quint32>(14965),static_cast<quint32>(17528),static_cast<quint32>(20368),static_cast<quint32>(24161),static_cast<quint32>(27694),static_cast<quint32>(31555),static_cast<quint32>(35759),static_cast<quint32>(40321),static_cast<quint32>(45255),static_cast<quint32>(50576),static_cast<quint32>(56299),static_cast<quint32>(62438),static_cast<quint32>(69008),static_cast<quint32>(77066),static_cast<quint32>(84643),static_cast<quint32>(92701),static_cast<quint32>(101255),static_cast<quint32>(110320),static_cast<quint32>(119910),static_cast<quint32>(130040),static_cast<quint32>(140725),static_cast<quint32>(151980),static_cast<quint32>(163820),static_cast<quint32>(178647),static_cast<quint32>(191877),static_cast<quint32>(205744),static_cast<quint32>(220264),static_cast<quint32>(235451),static_cast<quint32>(251321),static_cast<quint32>(267888),static_cast<quint32>(285168),static_cast<quint32>(303175),static_cast<quint32>(321925),static_cast<quint32>(341432),static_cast<quint32>(361712),static_cast<quint32>(382779),static_cast<quint32>(404649),static_cast<quint32>(427336),static_cast<quint32>(450856),static_cast<quint32>(475223),static_cast<quint32>(500453),static_cast<quint32>(526560),static_cast<quint32>(553560),static_cast<quint32>(581467),static_cast<quint32>(610297),static_cast<quint32>(640064),static_cast<quint32>(670784),static_cast<quint32>(702471),static_cast<quint32>(735141),static_cast<quint32>(768808),static_cast<quint32>(803488),static_cast<quint32>(839195),static_cast<quint32>(875945),static_cast<quint32>(913752),static_cast<quint32>(952632),static_cast<quint32>(992599),static_cast<quint32>(1033669),static_cast<quint32>(1075856),static_cast<quint32>(1119176),static_cast<quint32>(1163643),static_cast<quint32>(1209273),static_cast<quint32>(1256080),static_cast<quint32>(1304080),static_cast<quint32>(1371319),static_cast<quint32>(1422421),static_cast<quint32>(1474777),static_cast<quint32>(1528402),static_cast<quint32>(1583312),static_cast<quint32>(1639521),static_cast<quint32>(1697045),static_cast<quint32>(1755899),static_cast<quint32>(1816098),static_cast<quint32>(1877658),static_cast<quint32>(1940593),static_cast<quint32>(2004919),static_cast<quint32>(2070651),static_cast<quint32>(2137804),static_cast<quint32>(2206394),static_cast<quint32>(2276435),static_cast<quint32>(2347943),static_cast<quint32>(2420933)},
	{static_cast<quint32>(0),static_cast<quint32>(6),static_cast<quint32>(27),static_cast<quint32>(61),static_cast<quint32>(108),static_cast<quint32>(170),static_cast<quint32>(244),static_cast<quint32>(333),static_cast<quint32>(435),static_cast<quint32>(550),static_cast<quint32>(680),static_cast<quint32>(974),static_cast<quint32>(1022),static_cast<quint32>(1199),static_cast<quint32>(1391),static_cast<quint32>(1597),static_cast<quint32>(1817),static_cast<quint32>(2051),static_cast<quint32>(2300),static_cast<quint32>(2563),static_cast<quint32>(2840),static_cast<quint32>(3793),static_cast<quint32>(3533),static_cast<quint32>(3861),static_cast<quint32>(4204),static_cast<quint32>(4562),static_cast<quint32>(4934),static_cast<quint32>(5321),static_cast<quint32>(5723),static_cast<quint32>(6139),static_cast<quint32>(6570),static_cast<quint32>(8058),static_cast<quint32>(7577),static_cast<quint32>(8058),static_cast<quint32>(8554),static_cast<quint32>(9065),static_cast<quint32>(9590),static_cast<quint32>(10130),static_cast<quint32>(10685),static_cast<quint32>(11255),static_cast<quint32>(11840),static_cast<quint32>(14827),static_cast<quint32>(13230),static_cast<quint32>(13867),static_cast<quint32>(14520),static_cast<quint32>(15187),static_cast<quint32>(15870),static_cast<quint32>(16567),static_cast<quint32>(17280),static_cast<quint32>(18007),static_cast<quint32>(18750),static_cast<quint32>(19507),static_cast<quint32>(20280),static_cast<quint32>(21067),static_cast<quint32>(21870),static_cast<quint32>(22687),static_cast<quint32>(23520),static_cast<quint32>(24367),static_cast<quint32>(25230),static_cast<quint32>(26107),static_cast<quint32>(27000),static_cast<quint32>(27907),static_cast<quint32>(28830),static_cast<quint32>(29767),static_cast<quint32>(30720),static_cast<quint32>(31687),static_cast<quint32>(32670),static_cast<quint32>(33667),static_cast<quint32>(34680),static_cast<quint32>(35707),static_cast<quint32>(36750),static_cast<quint32>(37807),static_cast<quint32>(38880),static_cast<quint32>(39967),static_cast<quint32>(41070),static_cast<quint32>(42187),static_cast<quint32>(43320),static_cast<quint32>(44467),static_cast<quint32>(45630),static_cast<quint32>(46807),static_cast<quint32>(48000),static_cast<quint32>(67239),static_cast<quint32>(51102),static_cast<quint32>(52356),static_cast<quint32>(53625),static_cast<quint32>(54910),static_cast<quint32>(56209),static_cast<quint32>(57524),static_cast<quint32>(58854),static_cast<quint32>(60199),static_cast<quint32>(61560),static_cast<quint32>(62935),static_cast<quint32>(64326),static_cast<quint32>(65732),static_cast<quint32>(67153),static_cast<quint32>(68590),static_cast<quint32>(70041),static_cast<quint32>(71508),static_cast<quint32>(72990)},
	{6,18,16,9,25},{10,0,-28,-58,-98,-98,-26,136},{60,70,84,94,104,104,92,72},{200,0,-520,-1520,-2520,-3000,-2160,-80},{19,38,64,96,121,131,117,92},{14,15,15,16,17,17,17,20},{20,10,10,9,9,10,10,6}},

	{3,190,11,62,QT_TRANSLATE_NOOP("Char_Names","Aerith"),QString(":/characters/aerith"),
	QStringList()<<QT_TRANSLATE_NOOP("Char_Limits","Healing Wind")<<QT_TRANSLATE_NOOP("Char_Limits","Seal Evil")<<QT_TRANSLATE_NOOP("Char_Limits","Breath of the Earth")<<QT_TRANSLATE_NOOP("Char_Limits","Fury Brand")<<QT_TRANSLATE_NOOP("Char_Limits","Planet Protector")<<QT_TRANSLATE_NOOP("Char_Limits","Pulse of Life")<<QT_TRANSLATE_NOOP("Char_Limits","Great Gospel"),
	{static_cast<quint32>(0),static_cast<quint32>(6),static_cast<quint32>(32),static_cast<quint32>(92),static_cast<quint32>(199),static_cast<quint32>(366),static_cast<quint32>(607),static_cast<quint32>(935),static_cast<quint32>(1363),static_cast<quint32>(1905),static_cast<quint32>(2575),static_cast<quint32>(3542),static_cast<quint32>(4550),static_cast<quint32>(5733),static_cast<quint32>(7105),static_cast<quint32>(8680),static_cast<quint32>(10472),static_cast<quint32>(12495),static_cast<quint32>(14763),static_cast<quint32>(17290),static_cast<quint32>(20090),static_cast<quint32>(23831),static_cast<quint32>(27315),static_cast<quint32>(31123),static_cast<quint32>(35270),static_cast<quint32>(39770),static_cast<quint32>(44637),static_cast<quint32>(49885),static_cast<quint32>(55529),static_cast<quint32>(61584),static_cast<quint32>(68064),static_cast<quint32>(77066),static_cast<quint32>(84643),static_cast<quint32>(92701),static_cast<quint32>(101255),static_cast<quint32>(110320),static_cast<quint32>(119910),static_cast<quint32>(130040),static_cast<quint32>(140725),static_cast<quint32>(151980),static_cast<quint32>(163820),static_cast<quint32>(176259),static_cast<quint32>(189312),static_cast<quint32>(202994),static_cast<quint32>(217320),static_cast<quint32>(232305),static_cast<quint32>(247963),static_cast<quint32>(264309),static_cast<quint32>(281358),static_cast<quint32>(299125),static_cast<quint32>(317625),static_cast<quint32>(341432),static_cast<quint32>(361712),static_cast<quint32>(382779),static_cast<quint32>(404649),static_cast<quint32>(427336),static_cast<quint32>(450856),static_cast<quint32>(475223),static_cast<quint32>(500453),static_cast<quint32>(526560),static_cast<quint32>(553560),static_cast<quint32>(589211),static_cast<quint32>(618425),static_cast<quint32>(648589),static_cast<quint32>(679718),static_cast<quint32>(711828),static_cast<quint32>(744933),static_cast<quint32>(779049),static_cast<quint32>(814191),static_cast<quint32>(850374),static_cast<quint32>(887614),static_cast<quint32>(925925),static_cast<quint32>(965323),static_cast<quint32>(1005823),static_cast<quint32>(1047440),static_cast<quint32>(1090190),static_cast<quint32>(1134087),static_cast<quint32>(1179147),static_cast<quint32>(1225385),static_cast<quint32>(1272816),static_cast<quint32>(1321456),static_cast<quint32>(1407407),static_cast<quint32>(1459854),static_cast<quint32>(1513588),static_cast<quint32>(1568624),static_cast<quint32>(1624979),static_cast<quint32>(1682667),static_cast<quint32>(1741705),static_cast<quint32>(1802108),static_cast<quint32>(1863891),static_cast<quint32>(1927071),static_cast<quint32>(1991662),static_cast<quint32>(2057681),static_cast<quint32>(2125143),static_cast<quint32>(2194063),static_cast<quint32>(2264458),static_cast<quint32>(2336342),static_cast<quint32>(2409732),static_cast<quint32>(2484643)},
	{static_cast<quint32>(0),static_cast<quint32>(6),static_cast<quint32>(26),static_cast<quint32>(60),static_cast<quint32>(107),static_cast<quint32>(167),static_cast<quint32>(241),static_cast<quint32>(328),static_cast<quint32>(428),static_cast<quint32>(542),static_cast<quint32>(670),static_cast<quint32>(967),static_cast<quint32>(1008),static_cast<quint32>(1183),static_cast<quint32>(1372),static_cast<quint32>(1575),static_cast<quint32>(1792),static_cast<quint32>(2023),static_cast<quint32>(2268),static_cast<quint32>(2527),static_cast<quint32>(2800),static_cast<quint32>(3741),static_cast<quint32>(3484),static_cast<quint32>(3808),static_cast<quint32>(4147),static_cast<quint32>(4500),static_cast<quint32>(4867),static_cast<quint32>(5248),static_cast<quint32>(5644),static_cast<quint32>(6055),static_cast<quint32>(6480),static_cast<quint32>(9002),static_cast<quint32>(7577),static_cast<quint32>(8058),static_cast<quint32>(8554),static_cast<quint32>(9065),static_cast<quint32>(9590),static_cast<quint32>(10130),static_cast<quint32>(10685),static_cast<quint32>(11255),static_cast<quint32>(11840),static_cast<quint32>(12439),static_cast<quint32>(13053),static_cast<quint32>(13682),static_cast<quint32>(14326),static_cast<quint32>(14985),static_cast<quint32>(15658),static_cast<quint32>(16346),static_cast<quint32>(17049),static_cast<quint32>(17767),static_cast<quint32>(18500),static_cast<quint32>(23807),static_cast<quint32>(20280),static_cast<quint32>(21067),static_cast<quint32>(21870),static_cast<quint32>(22687),static_cast<quint32>(23520),static_cast<quint32>(24367),static_cast<quint32>(25230),static_cast<quint32>(26107),static_cast<quint32>(27000),static_cast<quint32>(35651),static_cast<quint32>(29214),static_cast<quint32>(30164),static_cast<quint32>(31129),static_cast<quint32>(32110),static_cast<quint32>(33105),static_cast<quint32>(34116),static_cast<quint32>(35142),static_cast<quint32>(36183),static_cast<quint32>(37240),static_cast<quint32>(38311),static_cast<quint32>(39398),static_cast<quint32>(40500),static_cast<quint32>(41617),static_cast<quint32>(42750),static_cast<quint32>(43897),static_cast<quint32>(45060),static_cast<quint32>(46238),static_cast<quint32>(47431),static_cast<quint32>(48640),static_cast<quint32>(85951),static_cast<quint32>(52447),static_cast<quint32>(53734),static_cast<quint32>(55036),static_cast<quint32>(56355),static_cast<quint32>(57688),static_cast<quint32>(59038),static_cast<quint32>(60403),static_cast<quint32>(61783),static_cast<quint32>(63180),static_cast<quint32>(64591),static_cast<quint32>(66019),static_cast<quint32>(67462),static_cast<quint32>(68920),static_cast<quint32>(70395),static_cast<quint32>(71884),static_cast<quint32>(73390),static_cast<quint32>(74911)},
	{23,20,0,1,28},{16,0,-30,-68,-116,-96,-6,188},{70,84,99,112,124,120,105,82},{160,0,-560,-1400,-2240,-2880,-2080,-400},{17,36,65,93,114,126,113,93},{14,15,15,16,16,18,17,17},{18,8,8,8,8,5,7,7}},

	{4,176,14,48,QT_TRANSLATE_NOOP("Char_Names","Red XIII"),QString(":/characters/red"),
	QStringList()<<QT_TRANSLATE_NOOP("Char_Limits","Sled Fang")<<QT_TRANSLATE_NOOP("Char_Limits","Lunatic High")<<QT_TRANSLATE_NOOP("Char_Limits","Blood Fang")<<QT_TRANSLATE_NOOP("Char_Limits","Stardust Ray")<<QT_TRANSLATE_NOOP("Char_Limits","Howling Moon")<<QT_TRANSLATE_NOOP("Char_Limits","Earth Rave")<<QT_TRANSLATE_NOOP("Char_Limits","Cosmo Memory"),
	 {static_cast<quint32>(0),static_cast<quint32>(6),static_cast<quint32>(33),static_cast<quint32>(94),static_cast<quint32>(202),static_cast<quint32>(372),static_cast<quint32>(616),static_cast<quint32>(949),static_cast<quint32>(1384),static_cast<quint32>(1934),static_cast<quint32>(2614),static_cast<quint32>(3588),static_cast<quint32>(4610),static_cast<quint32>(5809),static_cast<quint32>(7200),static_cast<quint32>(8797),static_cast<quint32>(10614),static_cast<quint32>(12665),static_cast<quint32>(14965),static_cast<quint32>(17528),static_cast<quint32>(20368),static_cast<quint32>(24493),static_cast<quint32>(28074),static_cast<quint32>(31988),static_cast<quint32>(36250),static_cast<quint32>(40875),static_cast<quint32>(45877),static_cast<quint32>(51271),static_cast<quint32>(57072),static_cast<quint32>(63295),static_cast<quint32>(69955),static_cast<quint32>(78112),static_cast<quint32>(85792),static_cast<quint32>(93959),static_cast<quint32>(102629),static_cast<quint32>(111816),static_cast<quint32>(121536),static_cast<quint32>(131803),static_cast<quint32>(142633),static_cast<quint32>(154040),static_cast<quint32>(166040),static_cast<quint32>(178647),static_cast<quint32>(191877),static_cast<quint32>(205744),static_cast<quint32>(220264),static_cast<quint32>(235451),static_cast<quint32>(251321),static_cast<quint32>(267888),static_cast<quint32>(285168),static_cast<quint32>(303175),static_cast<quint32>(321925),static_cast<quint32>(341432),static_cast<quint32>(361712),static_cast<quint32>(382779),static_cast<quint32>(404649),static_cast<quint32>(427336),static_cast<quint32>(450856),static_cast<quint32>(475223),static_cast<quint32>(500453),static_cast<quint32>(526560),static_cast<quint32>(553560),static_cast<quint32>(589211),static_cast<quint32>(618425),static_cast<quint32>(648589),static_cast<quint32>(679718),static_cast<quint32>(711828),static_cast<quint32>(744933),static_cast<quint32>(779049),static_cast<quint32>(814191),static_cast<quint32>(850374),static_cast<quint32>(887614),static_cast<quint32>(925925),static_cast<quint32>(965323),static_cast<quint32>(1005823),static_cast<quint32>(1047440),static_cast<quint32>(1090190),static_cast<quint32>(1134087),static_cast<quint32>(1179147),static_cast<quint32>(1225385),static_cast<quint32>(1272816),static_cast<quint32>(1321456),static_cast<quint32>(1371319),static_cast<quint32>(1422421),static_cast<quint32>(1474777),static_cast<quint32>(1528402),static_cast<quint32>(1583312),static_cast<quint32>(1639521),static_cast<quint32>(1697045),static_cast<quint32>(1755899),static_cast<quint32>(1816098),static_cast<quint32>(1877658),static_cast<quint32>(1940593),static_cast<quint32>(2004919),static_cast<quint32>(2070651),static_cast<quint32>(2137804),static_cast<quint32>(2206394),static_cast<quint32>(2276435),static_cast<quint32>(2347943),static_cast<quint32>(2420933)},
	 {static_cast<quint32>(0),static_cast<quint32>(6),static_cast<quint32>(27),static_cast<quint32>(61),static_cast<quint32>(108),static_cast<quint32>(170),static_cast<quint32>(244),static_cast<quint32>(333),static_cast<quint32>(435),static_cast<quint32>(550),static_cast<quint32>(680),static_cast<quint32>(974),static_cast<quint32>(1022),static_cast<quint32>(1199),static_cast<quint32>(1391),static_cast<quint32>(1597),static_cast<quint32>(1817),static_cast<quint32>(2051),static_cast<quint32>(2300),static_cast<quint32>(2563),static_cast<quint32>(2840),static_cast<quint32>(4125),static_cast<quint32>(3581),static_cast<quint32>(3914),static_cast<quint32>(4262),static_cast<quint32>(4625),static_cast<quint32>(5002),static_cast<quint32>(5394),static_cast<quint32>(5801),static_cast<quint32>(6223),static_cast<quint32>(6660),static_cast<quint32>(8157),static_cast<quint32>(7680),static_cast<quint32>(8167),static_cast<quint32>(8670),static_cast<quint32>(9187),static_cast<quint32>(9720),static_cast<quint32>(10267),static_cast<quint32>(10830),static_cast<quint32>(11407),static_cast<quint32>(12000),static_cast<quint32>(12607),static_cast<quint32>(13230),static_cast<quint32>(13867),static_cast<quint32>(14520),static_cast<quint32>(15187),static_cast<quint32>(15870),static_cast<quint32>(16567),static_cast<quint32>(17280),static_cast<quint32>(18007),static_cast<quint32>(18750),static_cast<quint32>(19507),static_cast<quint32>(20280),static_cast<quint32>(21067),static_cast<quint32>(21870),static_cast<quint32>(22687),static_cast<quint32>(23520),static_cast<quint32>(24367),static_cast<quint32>(25230),static_cast<quint32>(26107),static_cast<quint32>(27000),static_cast<quint32>(35651),static_cast<quint32>(29214),static_cast<quint32>(30164),static_cast<quint32>(31129),static_cast<quint32>(32110),static_cast<quint32>(33105),static_cast<quint32>(34116),static_cast<quint32>(35142),static_cast<quint32>(36183),static_cast<quint32>(37240),static_cast<quint32>(38311),static_cast<quint32>(39398),static_cast<quint32>(40500),static_cast<quint32>(41617),static_cast<quint32>(42750),static_cast<quint32>(43897),static_cast<quint32>(45060),static_cast<quint32>(46238),static_cast<quint32>(47431),static_cast<quint32>(48640),static_cast<quint32>(49863),static_cast<quint32>(51102),static_cast<quint32>(52356),static_cast<quint32>(53625),static_cast<quint32>(54910),static_cast<quint32>(56209),static_cast<quint32>(57524),static_cast<quint32>(58854),static_cast<quint32>(60199),static_cast<quint32>(61560),static_cast<quint32>(62935),static_cast<quint32>(64326),static_cast<quint32>(65732),static_cast<quint32>(67153),static_cast<quint32>(68590),static_cast<quint32>(70041),static_cast<quint32>(71508),static_cast<quint32>(72990)},
	{12,11,13,9,23},{12,-6,-28,-60,-104,-126,-16,210},{58,75,86,97,108,112,94,66},{200,-40,-640,-1560,-2360,-2760,-1840,-80},{21,45,75,105,126,134,119,97},{14,15,15,16,16,18,17,17},{18,8,8,8,8,5,7,7}},

	{5,215,14,87,QT_TRANSLATE_NOOP("Char_Names","Yuffie"),QString(":/characters/yuffie"),
	QStringList()<<QT_TRANSLATE_NOOP("Char_Limits","Greased Lightning")<<QT_TRANSLATE_NOOP("Char_Limits","Clear Tranquil")<<QT_TRANSLATE_NOOP("Char_Limits","Landscaper")<<QT_TRANSLATE_NOOP("Char_Limits","Bloodfest")<<QT_TRANSLATE_NOOP("Char_Limits","Gauntlet")<<QT_TRANSLATE_NOOP("Char_Limits","Doom of the Living")<<QT_TRANSLATE_NOOP("Char_Limits","All Creation"),
	 {static_cast<quint32>(0),static_cast<quint32>(6),static_cast<quint32>(33),static_cast<quint32>(95),static_cast<quint32>(205),static_cast<quint32>(377),static_cast<quint32>(625),static_cast<quint32>(963),static_cast<quint32>(1404),static_cast<quint32>(1962),static_cast<quint32>(2652),static_cast<quint32>(3639),static_cast<quint32>(4675),static_cast<quint32>(5891),static_cast<quint32>(7302),static_cast<quint32>(8922),static_cast<quint32>(10765),static_cast<quint32>(12845),static_cast<quint32>(15177),static_cast<quint32>(17776),static_cast<quint32>(20656),static_cast<quint32>(24827),static_cast<quint32>(28457),static_cast<quint32>(32424),static_cast<quint32>(36744),static_cast<quint32>(41431),static_cast<quint32>(46501),static_cast<quint32>(51968),static_cast<quint32>(57848),static_cast<quint32>(64155),static_cast<quint32>(70905),static_cast<quint32>(78112),static_cast<quint32>(85792),static_cast<quint32>(93959),static_cast<quint32>(102629),static_cast<quint32>(111816),static_cast<quint32>(121536),static_cast<quint32>(131803),static_cast<quint32>(142633),static_cast<quint32>(154040),static_cast<quint32>(166040),static_cast<quint32>(181023),static_cast<quint32>(194429),static_cast<quint32>(208481),static_cast<quint32>(223194),static_cast<quint32>(238584),static_cast<quint32>(254665),static_cast<quint32>(271453),static_cast<quint32>(288963),static_cast<quint32>(307210),static_cast<quint32>(326210),static_cast<quint32>(345977),static_cast<quint32>(366527),static_cast<quint32>(387875),static_cast<quint32>(410036),static_cast<quint32>(433026),static_cast<quint32>(456859),static_cast<quint32>(481551),static_cast<quint32>(507117),static_cast<quint32>(533572),static_cast<quint32>(560932),static_cast<quint32>(596961),static_cast<quint32>(626559),static_cast<quint32>(657120),static_cast<quint32>(688659),static_cast<quint32>(721191),static_cast<quint32>(754732),static_cast<quint32>(789297),static_cast<quint32>(824901),static_cast<quint32>(861560),static_cast<quint32>(899290),static_cast<quint32>(938105),static_cast<quint32>(978021),static_cast<quint32>(1019054),static_cast<quint32>(1061219),static_cast<quint32>(1104531),static_cast<quint32>(1149006),static_cast<quint32>(1194659),static_cast<quint32>(1241505),static_cast<quint32>(1289560),static_cast<quint32>(1338840),static_cast<quint32>(1389359),static_cast<quint32>(1441133),static_cast<quint32>(1494178),static_cast<quint32>(1548509),static_cast<quint32>(1604141),static_cast<quint32>(1661090),static_cast<quint32>(1719371),static_cast<quint32>(1778999),static_cast<quint32>(1839990),static_cast<quint32>(1902360),static_cast<quint32>(1966123),static_cast<quint32>(2031295),static_cast<quint32>(2097892),static_cast<quint32>(2165929),static_cast<quint32>(2235421),static_cast<quint32>(2306384),static_cast<quint32>(2378833),static_cast<quint32>(2452783)},
	 {static_cast<quint32>(0),static_cast<quint32>(6),static_cast<quint32>(27),static_cast<quint32>(62),static_cast<quint32>(110),static_cast<quint32>(172),static_cast<quint32>(248),static_cast<quint32>(338),static_cast<quint32>(441),static_cast<quint32>(558),static_cast<quint32>(690),static_cast<quint32>(987),static_cast<quint32>(1036),static_cast<quint32>(1216),static_cast<quint32>(1411),static_cast<quint32>(1620),static_cast<quint32>(1843),static_cast<quint32>(2080),static_cast<quint32>(2332),static_cast<quint32>(2599),static_cast<quint32>(2880),static_cast<quint32>(4171),static_cast<quint32>(3630),static_cast<quint32>(3967),static_cast<quint32>(4320),static_cast<quint32>(4687),static_cast<quint32>(5070),static_cast<quint32>(5467),static_cast<quint32>(5880),static_cast<quint32>(6307),static_cast<quint32>(6750),static_cast<quint32>(7207),static_cast<quint32>(7680),static_cast<quint32>(8167),static_cast<quint32>(8670),static_cast<quint32>(9187),static_cast<quint32>(9720),static_cast<quint32>(10267),static_cast<quint32>(10830),static_cast<quint32>(11407),static_cast<quint32>(12000),static_cast<quint32>(14983),static_cast<quint32>(13406),static_cast<quint32>(14052),static_cast<quint32>(14713),static_cast<quint32>(15390),static_cast<quint32>(16081),static_cast<quint32>(16788),static_cast<quint32>(17510),static_cast<quint32>(18247),static_cast<quint32>(19000),static_cast<quint32>(19767),static_cast<quint32>(20550),static_cast<quint32>(21348),static_cast<quint32>(22161),static_cast<quint32>(22990),static_cast<quint32>(23833),static_cast<quint32>(24692),static_cast<quint32>(25566),static_cast<quint32>(26455),static_cast<quint32>(27360),static_cast<quint32>(36029),static_cast<quint32>(29598),static_cast<quint32>(30561),static_cast<quint32>(31539),static_cast<quint32>(32532),static_cast<quint32>(33541),static_cast<quint32>(34565),static_cast<quint32>(35604),static_cast<quint32>(36659),static_cast<quint32>(37730),static_cast<quint32>(38815),static_cast<quint32>(39916),static_cast<quint32>(41033),static_cast<quint32>(42165),static_cast<quint32>(43312),static_cast<quint32>(44475),static_cast<quint32>(45653),static_cast<quint32>(46846),static_cast<quint32>(48055),static_cast<quint32>(49280),static_cast<quint32>(50519),static_cast<quint32>(51774),static_cast<quint32>(53045),static_cast<quint32>(54331),static_cast<quint32>(55632),static_cast<quint32>(56949),static_cast<quint32>(58281),static_cast<quint32>(59628),static_cast<quint32>(60991),static_cast<quint32>(62370),static_cast<quint32>(63763),static_cast<quint32>(65172),static_cast<quint32>(66597),static_cast<quint32>(68037),static_cast<quint32>(69492),static_cast<quint32>(70963),static_cast<quint32>(72449),static_cast<quint32>(73950)},
	{16,19,11,10,24},{10,-2,-20,-58,-110,-130,20,126},{58,72,80,93,106,110,85,72},{200,0,-560,-1320,-2160,-2960,-2560,-520},{18,37,64,89,111,127,120,96},{16,17,17,17,18,18,18,19},{20,10,10,10,8,10,10,9}},

	{6,229,13,101,QT_TRANSLATE_NOOP("Char_Names","Cait Sith"),QString(":/characters/cait"),
	QStringList()<<QT_TRANSLATE_NOOP("Char_Limits","Dice")<<QT_TRANSLATE_NOOP("Char_Limits","")<<QT_TRANSLATE_NOOP("Char_Limits","Slots")<<QT_TRANSLATE_NOOP("Char_Limits","")<<QT_TRANSLATE_NOOP("Char_Limits","")<<QT_TRANSLATE_NOOP("Char_Limits","")<<QT_TRANSLATE_NOOP("Char_Limits",""),
	{static_cast<quint32>(0),static_cast<quint32>(6),static_cast<quint32>(33),static_cast<quint32>(95),static_cast<quint32>(205),static_cast<quint32>(377),static_cast<quint32>(625),static_cast<quint32>(963),static_cast<quint32>(1404),static_cast<quint32>(1962),static_cast<quint32>(2652),static_cast<quint32>(3639),static_cast<quint32>(4675),static_cast<quint32>(5891),static_cast<quint32>(7302),static_cast<quint32>(8922),static_cast<quint32>(10765),static_cast<quint32>(12845),static_cast<quint32>(15177),static_cast<quint32>(17776),static_cast<quint32>(20656),static_cast<quint32>(24827),static_cast<quint32>(28457),static_cast<quint32>(32424),static_cast<quint32>(36744),static_cast<quint32>(41431),static_cast<quint32>(46501),static_cast<quint32>(51968),static_cast<quint32>(57848),static_cast<quint32>(64155),static_cast<quint32>(70905),static_cast<quint32>(78112),static_cast<quint32>(85792),static_cast<quint32>(93959),static_cast<quint32>(102629),static_cast<quint32>(111816),static_cast<quint32>(121536),static_cast<quint32>(131803),static_cast<quint32>(142633),static_cast<quint32>(154040),static_cast<quint32>(166040),static_cast<quint32>(178647),static_cast<quint32>(191877),static_cast<quint32>(205744),static_cast<quint32>(220264),static_cast<quint32>(235451),static_cast<quint32>(251321),static_cast<quint32>(267888),static_cast<quint32>(285168),static_cast<quint32>(303175),static_cast<quint32>(321925),static_cast<quint32>(345977),static_cast<quint32>(366527),static_cast<quint32>(387875),static_cast<quint32>(410036),static_cast<quint32>(433026),static_cast<quint32>(456859),static_cast<quint32>(481551),static_cast<quint32>(507117),static_cast<quint32>(533572),static_cast<quint32>(560932),static_cast<quint32>(589211),static_cast<quint32>(618425),static_cast<quint32>(648589),static_cast<quint32>(679718),static_cast<quint32>(711828),static_cast<quint32>(744933),static_cast<quint32>(779049),static_cast<quint32>(814191),static_cast<quint32>(850374),static_cast<quint32>(887614),static_cast<quint32>(925925),static_cast<quint32>(965323),static_cast<quint32>(1005823),static_cast<quint32>(1047440),static_cast<quint32>(1090190),static_cast<quint32>(1134087),static_cast<quint32>(1179147),static_cast<quint32>(1225385),static_cast<quint32>(1272816),static_cast<quint32>(1321456),static_cast<quint32>(1371319),static_cast<quint32>(1422421),static_cast<quint32>(1474777),static_cast<quint32>(1528402),static_cast<quint32>(1583312),static_cast<quint32>(1639521),static_cast<quint32>(1697045),static_cast<quint32>(1755899),static_cast<quint32>(1816098),static_cast<quint32>(1877658),static_cast<quint32>(1940593),static_cast<quint32>(2004919),static_cast<quint32>(2070651),static_cast<quint32>(2137804),static_cast<quint32>(2206394),static_cast<quint32>(2276435),static_cast<quint32>(2347943),static_cast<quint32>(2420933)},
	{static_cast<quint32>(0),static_cast<quint32>(6),static_cast<quint32>(27),static_cast<quint32>(62),static_cast<quint32>(110),static_cast<quint32>(172),static_cast<quint32>(248),static_cast<quint32>(338),static_cast<quint32>(441),static_cast<quint32>(558),static_cast<quint32>(690),static_cast<quint32>(987),static_cast<quint32>(1036),static_cast<quint32>(1216),static_cast<quint32>(1411),static_cast<quint32>(1620),static_cast<quint32>(1843),static_cast<quint32>(2080),static_cast<quint32>(2332),static_cast<quint32>(2599),static_cast<quint32>(2880),static_cast<quint32>(4171),static_cast<quint32>(3630),static_cast<quint32>(3967),static_cast<quint32>(4320),static_cast<quint32>(4687),static_cast<quint32>(5070),static_cast<quint32>(5467),static_cast<quint32>(5880),static_cast<quint32>(6307),static_cast<quint32>(6750),static_cast<quint32>(7207),static_cast<quint32>(7680),static_cast<quint32>(8167),static_cast<quint32>(8670),static_cast<quint32>(9187),static_cast<quint32>(9720),static_cast<quint32>(10267),static_cast<quint32>(10830),static_cast<quint32>(11407),static_cast<quint32>(12000),static_cast<quint32>(12607),static_cast<quint32>(13230),static_cast<quint32>(13867),static_cast<quint32>(14520),static_cast<quint32>(15187),static_cast<quint32>(15870),static_cast<quint32>(16567),static_cast<quint32>(17280),static_cast<quint32>(18007),static_cast<quint32>(18750),static_cast<quint32>(24052),static_cast<quint32>(20550),static_cast<quint32>(21348),static_cast<quint32>(22161),static_cast<quint32>(22990),static_cast<quint32>(23833),static_cast<quint32>(24692),static_cast<quint32>(25566),static_cast<quint32>(26455),static_cast<quint32>(27360),static_cast<quint32>(28279),static_cast<quint32>(29214),static_cast<quint32>(30164),static_cast<quint32>(31129),static_cast<quint32>(32110),static_cast<quint32>(33105),static_cast<quint32>(34116),static_cast<quint32>(35142),static_cast<quint32>(36183),static_cast<quint32>(37240),static_cast<quint32>(38311),static_cast<quint32>(39398),static_cast<quint32>(40500),static_cast<quint32>(41617),static_cast<quint32>(42750),static_cast<quint32>(43897),static_cast<quint32>(45060),static_cast<quint32>(46238),static_cast<quint32>(47431),static_cast<quint32>(48640),static_cast<quint32>(49863),static_cast<quint32>(51102),static_cast<quint32>(52356),static_cast<quint32>(53625),static_cast<quint32>(54910),static_cast<quint32>(56209),static_cast<quint32>(57524),static_cast<quint32>(58854),static_cast<quint32>(60199),static_cast<quint32>(61560),static_cast<quint32>(62935),static_cast<quint32>(64326),static_cast<quint32>(65732),static_cast<quint32>(67153),static_cast<quint32>(68590),static_cast<quint32>(70041),static_cast<quint32>(71508),static_cast<quint32>(72990)},
	{19,22,6,4,28},{12,-2,-20,-60,-104,-104,-20,178},{60,75,83,97,108,108,94,70},{200,-80,-640,-1640,-2760,-2600,-240,2000},{24,51,80,111,141,138,99,72},{14,15,15,16,17,17,17,20},{20,10,10,9,9,10,10,6}},

	{7,242,13,114,QT_TRANSLATE_NOOP("Char_Names","Vincent"),QString(":/characters/vincent"),
	QStringList()<<QT_TRANSLATE_NOOP("Char_Limits","Galian Beast")<<QT_TRANSLATE_NOOP("Char_Limits","")<<QT_TRANSLATE_NOOP("Char_Limits","Death Gigas")<<QT_TRANSLATE_NOOP("Char_Limits","")<<QT_TRANSLATE_NOOP("Char_Limits","Hellmasker")<<QT_TRANSLATE_NOOP("Char_Limits","")<<QT_TRANSLATE_NOOP("Char_Limits","Chaos"),
	{static_cast<quint32>(0),static_cast<quint32>(7),static_cast<quint32>(35),static_cast<quint32>(98),static_cast<quint32>(210),static_cast<quint32>(385),static_cast<quint32>(637),static_cast<quint32>(980),static_cast<quint32>(1428),static_cast<quint32>(1995),static_cast<quint32>(2695),static_cast<quint32>(3639),static_cast<quint32>(4675),static_cast<quint32>(5891),static_cast<quint32>(7302),static_cast<quint32>(8922),static_cast<quint32>(10765),static_cast<quint32>(12845),static_cast<quint32>(15177),static_cast<quint32>(17776),static_cast<quint32>(20656),static_cast<quint32>(24827),static_cast<quint32>(28457),static_cast<quint32>(32424),static_cast<quint32>(36744),static_cast<quint32>(41431),static_cast<quint32>(46501),static_cast<quint32>(51968),static_cast<quint32>(57848),static_cast<quint32>(64155),static_cast<quint32>(70905),static_cast<quint32>(79149),static_cast<quint32>(86931),static_cast<quint32>(95207),static_cast<quint32>(103992),static_cast<quint32>(113302),static_cast<quint32>(123151),static_cast<quint32>(133555),static_cast<quint32>(144529),static_cast<quint32>(156088),static_cast<quint32>(168248),static_cast<quint32>(181023),static_cast<quint32>(194429),static_cast<quint32>(208481),static_cast<quint32>(223194),static_cast<quint32>(238584),static_cast<quint32>(254665),static_cast<quint32>(271453),static_cast<quint32>(288963),static_cast<quint32>(307210),static_cast<quint32>(326210),static_cast<quint32>(345977),static_cast<quint32>(366527),static_cast<quint32>(387875),static_cast<quint32>(410036),static_cast<quint32>(433026),static_cast<quint32>(456859),static_cast<quint32>(481551),static_cast<quint32>(507117),static_cast<quint32>(533572),static_cast<quint32>(560932),static_cast<quint32>(589211),static_cast<quint32>(618425),static_cast<quint32>(648589),static_cast<quint32>(679718),static_cast<quint32>(711828),static_cast<quint32>(744933),static_cast<quint32>(779049),static_cast<quint32>(814191),static_cast<quint32>(850374),static_cast<quint32>(887614),static_cast<quint32>(925925),static_cast<quint32>(965323),static_cast<quint32>(1005823),static_cast<quint32>(1047440),static_cast<quint32>(1090190),static_cast<quint32>(1134087),static_cast<quint32>(1179147),static_cast<quint32>(1225385),static_cast<quint32>(1272816),static_cast<quint32>(1321456),static_cast<quint32>(1371319),static_cast<quint32>(1422421),static_cast<quint32>(1474777),static_cast<quint32>(1528402),static_cast<quint32>(1583312),static_cast<quint32>(1639521),static_cast<quint32>(1697045),static_cast<quint32>(1755899),static_cast<quint32>(1816098),static_cast<quint32>(1877658),static_cast<quint32>(1940593),static_cast<quint32>(2004919),static_cast<quint32>(2070651),static_cast<quint32>(2137804),static_cast<quint32>(2206394),static_cast<quint32>(2276435),static_cast<quint32>(2347943),static_cast<quint32>(2420933)},
	{static_cast<quint32>(0),static_cast<quint32>(7),static_cast<quint32>(28),static_cast<quint32>(63),static_cast<quint32>(112),static_cast<quint32>(175),static_cast<quint32>(252),static_cast<quint32>(343),static_cast<quint32>(448),static_cast<quint32>(567),static_cast<quint32>(700),static_cast<quint32>(944),static_cast<quint32>(1036),static_cast<quint32>(1216),static_cast<quint32>(1411),static_cast<quint32>(1620),static_cast<quint32>(1843),static_cast<quint32>(2080),static_cast<quint32>(2332),static_cast<quint32>(2599),static_cast<quint32>(2880),static_cast<quint32>(4171),static_cast<quint32>(3630),static_cast<quint32>(3967),static_cast<quint32>(4320),static_cast<quint32>(4687),static_cast<quint32>(5070),static_cast<quint32>(5467),static_cast<quint32>(5880),static_cast<quint32>(6307),static_cast<quint32>(6750),static_cast<quint32>(8244),static_cast<quint32>(7782),static_cast<quint32>(8276),static_cast<quint32>(8785),static_cast<quint32>(9310),static_cast<quint32>(9849),static_cast<quint32>(10404),static_cast<quint32>(10974),static_cast<quint32>(11559),static_cast<quint32>(12160),static_cast<quint32>(12775),static_cast<quint32>(13406),static_cast<quint32>(14052),static_cast<quint32>(14713),static_cast<quint32>(15390),static_cast<quint32>(16081),static_cast<quint32>(16788),static_cast<quint32>(17510),static_cast<quint32>(18247),static_cast<quint32>(19000),static_cast<quint32>(19767),static_cast<quint32>(20550),static_cast<quint32>(21348),static_cast<quint32>(22161),static_cast<quint32>(22990),static_cast<quint32>(23833),static_cast<quint32>(24692),static_cast<quint32>(25566),static_cast<quint32>(26455),static_cast<quint32>(27360),static_cast<quint32>(28279),static_cast<quint32>(29214),static_cast<quint32>(30164),static_cast<quint32>(31129),static_cast<quint32>(32110),static_cast<quint32>(33105),static_cast<quint32>(34116),static_cast<quint32>(35142),static_cast<quint32>(36183),static_cast<quint32>(37240),static_cast<quint32>(38311),static_cast<quint32>(39398),static_cast<quint32>(40500),static_cast<quint32>(41617),static_cast<quint32>(42750),static_cast<quint32>(43897),static_cast<quint32>(45060),static_cast<quint32>(46238),static_cast<quint32>(47431),static_cast<quint32>(48640),static_cast<quint32>(49863),static_cast<quint32>(51102),static_cast<quint32>(52356),static_cast<quint32>(53625),static_cast<quint32>(54910),static_cast<quint32>(56209),static_cast<quint32>(57524),static_cast<quint32>(58854),static_cast<quint32>(60199),static_cast<quint32>(61560),static_cast<quint32>(62935),static_cast<quint32>(64326),static_cast<quint32>(65732),static_cast<quint32>(67153),static_cast<quint32>(68590),static_cast<quint32>(70041),static_cast<quint32>(71508),static_cast<quint32>(72990)},
	{21,22,6,4,28},{12,-6,-26,-44,-60,-86,38,74},{63,80,90,96,100,105,97,84},{160,-80,-600,-1160,-2120,-2800,-2640,-400},{18,41,67,86,110,123,120,92},{14,15,15,16,17,17,17,20},{20,10,10,9,9,10,10,6}},

	{8,201,14,73,QT_TRANSLATE_NOOP("Char_Names","Cid"),QString(":/characters/cid"),
	QStringList()<<QT_TRANSLATE_NOOP("Char_Limits","Boost Jump")<<QT_TRANSLATE_NOOP("Char_Limits","Dynamite") <<QT_TRANSLATE_NOOP("Char_Limits","Hyper Jump")<<QT_TRANSLATE_NOOP("Char_Limits","Dragon")<<QT_TRANSLATE_NOOP("Char_Limits","Dragon Dive")<<QT_TRANSLATE_NOOP("Char_Limits","Big Brawl")<<QT_TRANSLATE_NOOP("Char_Limits","Highwind"),
	 {static_cast<quint32>(0),static_cast<quint32>(6),static_cast<quint32>(33),static_cast<quint32>(95),static_cast<quint32>(205),static_cast<quint32>(377),static_cast<quint32>(625),static_cast<quint32>(963),static_cast<quint32>(1404),static_cast<quint32>(1962),static_cast<quint32>(2652),static_cast<quint32>(3639),static_cast<quint32>(4675),static_cast<quint32>(5891),static_cast<quint32>(7302),static_cast<quint32>(8922),static_cast<quint32>(10765),static_cast<quint32>(12845),static_cast<quint32>(15177),static_cast<quint32>(17776),static_cast<quint32>(20656),static_cast<quint32>(24827),static_cast<quint32>(28457),static_cast<quint32>(32424),static_cast<quint32>(36744),static_cast<quint32>(41431),static_cast<quint32>(46501),static_cast<quint32>(51968),static_cast<quint32>(57848),static_cast<quint32>(64155),static_cast<quint32>(70905),static_cast<quint32>(78112),static_cast<quint32>(85792),static_cast<quint32>(93959),static_cast<quint32>(102629),static_cast<quint32>(111816),static_cast<quint32>(121536),static_cast<quint32>(131803),static_cast<quint32>(142633),static_cast<quint32>(154040),static_cast<quint32>(166040),static_cast<quint32>(181023),static_cast<quint32>(194429),static_cast<quint32>(208481),static_cast<quint32>(223194),static_cast<quint32>(238584),static_cast<quint32>(254665),static_cast<quint32>(271453),static_cast<quint32>(288963),static_cast<quint32>(307210),static_cast<quint32>(326210),static_cast<quint32>(345977),static_cast<quint32>(366527),static_cast<quint32>(387875),static_cast<quint32>(410036),static_cast<quint32>(433026),static_cast<quint32>(456859),static_cast<quint32>(481551),static_cast<quint32>(507117),static_cast<quint32>(533572),static_cast<quint32>(560932),static_cast<quint32>(596961),static_cast<quint32>(626559),static_cast<quint32>(657120),static_cast<quint32>(688659),static_cast<quint32>(721191),static_cast<quint32>(754732),static_cast<quint32>(789297),static_cast<quint32>(824901),static_cast<quint32>(861560),static_cast<quint32>(899290),static_cast<quint32>(938105),static_cast<quint32>(978021),static_cast<quint32>(1019054),static_cast<quint32>(1061219),static_cast<quint32>(1104531),static_cast<quint32>(1149006),static_cast<quint32>(1194659),static_cast<quint32>(1241505),static_cast<quint32>(1289560),static_cast<quint32>(1338840),static_cast<quint32>(1389359),static_cast<quint32>(1441133),static_cast<quint32>(1494178),static_cast<quint32>(1548509),static_cast<quint32>(1604141),static_cast<quint32>(1661090),static_cast<quint32>(1719371),static_cast<quint32>(1778999),static_cast<quint32>(1839990),static_cast<quint32>(1902360),static_cast<quint32>(1966123),static_cast<quint32>(2031295),static_cast<quint32>(2097892),static_cast<quint32>(2165929),static_cast<quint32>(2235421),static_cast<quint32>(2306384),static_cast<quint32>(2378833),static_cast<quint32>(2452783)},
	 {static_cast<quint32>(0),static_cast<quint32>(6),static_cast<quint32>(27),static_cast<quint32>(62),static_cast<quint32>(110),static_cast<quint32>(172),static_cast<quint32>(248),static_cast<quint32>(338),static_cast<quint32>(441),static_cast<quint32>(558),static_cast<quint32>(690),static_cast<quint32>(987),static_cast<quint32>(1036),static_cast<quint32>(1216),static_cast<quint32>(1411),static_cast<quint32>(1620),static_cast<quint32>(1843),static_cast<quint32>(2080),static_cast<quint32>(2332),static_cast<quint32>(2599),static_cast<quint32>(2880),static_cast<quint32>(4171),static_cast<quint32>(3630),static_cast<quint32>(3967),static_cast<quint32>(4320),static_cast<quint32>(4687),static_cast<quint32>(5070),static_cast<quint32>(5467),static_cast<quint32>(5880),static_cast<quint32>(6307),static_cast<quint32>(6750),static_cast<quint32>(7207),static_cast<quint32>(7680),static_cast<quint32>(8167),static_cast<quint32>(8670),static_cast<quint32>(9187),static_cast<quint32>(9720),static_cast<quint32>(10267),static_cast<quint32>(10830),static_cast<quint32>(11407),static_cast<quint32>(12000),static_cast<quint32>(14983),static_cast<quint32>(13406),static_cast<quint32>(14052),static_cast<quint32>(14713),static_cast<quint32>(15390),static_cast<quint32>(16081),static_cast<quint32>(16788),static_cast<quint32>(17510),static_cast<quint32>(18247),static_cast<quint32>(19000),static_cast<quint32>(19767),static_cast<quint32>(20550),static_cast<quint32>(21348),static_cast<quint32>(22161),static_cast<quint32>(22990),static_cast<quint32>(23833),static_cast<quint32>(24692),static_cast<quint32>(25566),static_cast<quint32>(26455),static_cast<quint32>(27360),static_cast<quint32>(36029),static_cast<quint32>(29598),static_cast<quint32>(30561),static_cast<quint32>(31539),static_cast<quint32>(32532),static_cast<quint32>(33541),static_cast<quint32>(34565),static_cast<quint32>(35604),static_cast<quint32>(36659),static_cast<quint32>(37730),static_cast<quint32>(38815),static_cast<quint32>(39916),static_cast<quint32>(41033),static_cast<quint32>(42165),static_cast<quint32>(43312),static_cast<quint32>(44475),static_cast<quint32>(45653),static_cast<quint32>(46846),static_cast<quint32>(48055),static_cast<quint32>(49280),static_cast<quint32>(50519),static_cast<quint32>(51774),static_cast<quint32>(53045),static_cast<quint32>(54331),static_cast<quint32>(55632),static_cast<quint32>(56949),static_cast<quint32>(58281),static_cast<quint32>(59628),static_cast<quint32>(60991),static_cast<quint32>(62370),static_cast<quint32>(63763),static_cast<quint32>(65172),static_cast<quint32>(66597),static_cast<quint32>(68037),static_cast<quint32>(69492),static_cast<quint32>(70963),static_cast<quint32>(72449),static_cast<quint32>(73950)},
	 {11,7,17,15,27},{10,-12,-26,-38,-66,-116,69,140},{54,75,83,87,94,104,89,69},{200,-40,-640,-1640,-2360,-2560,-1720,-0},{23,44,73,107,125,129,115,93},{14,15,15,15,16,17,18,20},{15,8,8,7,6,5,4,3}},

	{9,128,16,0,QT_TRANSLATE_NOOP("Char_Names","Y.Cloud"),QString(":/characters/y_cloud"),
	QStringList()<<QT_TRANSLATE_NOOP("Char_Limits","Braver")<<QT_TRANSLATE_NOOP("Char_Limits","Cross-Slash")<<QT_TRANSLATE_NOOP("Char_Limits","Blade Beam")<<QT_TRANSLATE_NOOP("Char_Limits","Climhazzard")<<QT_TRANSLATE_NOOP("Char_Limits","Meteorain")<<QT_TRANSLATE_NOOP("Char_Limits","Finishing Touch")<<QT_TRANSLATE_NOOP("Char_Limits","Omnislash"),
	 {static_cast<quint32>(0),static_cast<quint32>(13),static_cast<quint32>(65),static_cast<quint32>(182),static_cast<quint32>(390),static_cast<quint32>(715),static_cast<quint32>(1183),static_cast<quint32>(1820),static_cast<quint32>(2652),static_cast<quint32>(3705),static_cast<quint32>(5005),static_cast<quint32>(7084),static_cast<quint32>(9100),static_cast<quint32>(11466),static_cast<quint32>(14210),static_cast<quint32>(17360),static_cast<quint32>(20944),static_cast<quint32>(24990),static_cast<quint32>(29526),static_cast<quint32>(34580),static_cast<quint32>(40180),static_cast<quint32>(46354),static_cast<quint32>(53130),static_cast<quint32>(60536),static_cast<quint32>(68600),static_cast<quint32>(77350),static_cast<quint32>(86814),static_cast<quint32>(97020),static_cast<quint32>(107996),static_cast<quint32>(119770),static_cast<quint32>(132370),static_cast<quint32>(114576),static_cast<quint32>(125840),static_cast<quint32>(137819),static_cast<quint32>(150535),static_cast<quint32>(164010),static_cast<quint32>(178266),static_cast<quint32>(193325),static_cast<quint32>(209209),static_cast<quint32>(225940),static_cast<quint32>(243540),static_cast<quint32>(214389),static_cast<quint32>(230265),static_cast<quint32>(246906),static_cast<quint32>(264330),static_cast<quint32>(282555),static_cast<quint32>(301599),static_cast<quint32>(321480),static_cast<quint32>(342216),static_cast<quint32>(363825),static_cast<quint32>(386325),static_cast<quint32>(318682),static_cast<quint32>(337610),static_cast<quint32>(357273),static_cast<quint32>(377685),static_cast<quint32>(398860),static_cast<quint32>(420812),static_cast<quint32>(443555),static_cast<quint32>(467103),static_cast<quint32>(491470),static_cast<quint32>(516670),static_cast<quint32>(372124),static_cast<quint32>(390575),static_cast<quint32>(409626),static_cast<quint32>(429286),static_cast<quint32>(449566),static_cast<quint32>(470474),static_cast<quint32>(492021),static_cast<quint32>(514216),static_cast<quint32>(537068),static_cast<quint32>(560588),static_cast<quint32>(584784),static_cast<quint32>(609667),static_cast<quint32>(635246),static_cast<quint32>(661530),static_cast<quint32>(688530),static_cast<quint32>(716254),static_cast<quint32>(744713),static_cast<quint32>(773916),static_cast<quint32>(803872),static_cast<quint32>(834592),static_cast<quint32>(487154),static_cast<quint32>(505308),static_cast<quint32>(523908),static_cast<quint32>(542959),static_cast<quint32>(562466),static_cast<quint32>(582435),static_cast<quint32>(602871),static_cast<quint32>(623779),static_cast<quint32>(645165),static_cast<quint32>(667035),static_cast<quint32>(689393),static_cast<quint32>(712245),static_cast<quint32>(735597),static_cast<quint32>(759454),static_cast<quint32>(783821),static_cast<quint32>(808704),static_cast<quint32>(834108),static_cast<quint32>(860038)},
	 {static_cast<quint32>(0),static_cast<quint32>(13),static_cast<quint32>(52),static_cast<quint32>(117),static_cast<quint32>(208),static_cast<quint32>(325),static_cast<quint32>(468),static_cast<quint32>(637),static_cast<quint32>(832),static_cast<quint32>(1053),static_cast<quint32>(1300),static_cast<quint32>(2079),static_cast<quint32>(2016),static_cast<quint32>(2366),static_cast<quint32>(2744),static_cast<quint32>(3150),static_cast<quint32>(3584),static_cast<quint32>(4046),static_cast<quint32>(4536),static_cast<quint32>(5054),static_cast<quint32>(5600),static_cast<quint32>(6174),static_cast<quint32>(6776),static_cast<quint32>(7406),static_cast<quint32>(8064),static_cast<quint32>(8750),static_cast<quint32>(9464),static_cast<quint32>(10206),static_cast<quint32>(10976),static_cast<quint32>(11774),static_cast<quint32>(12600),static_cast<quint32>(-17794),static_cast<quint32>(11264),static_cast<quint32>(11979),static_cast<quint32>(12716),static_cast<quint32>(13475),static_cast<quint32>(14256),static_cast<quint32>(15059),static_cast<quint32>(15884),static_cast<quint32>(16731),static_cast<quint32>(17600),static_cast<quint32>(-29151),static_cast<quint32>(15876),static_cast<quint32>(16641),static_cast<quint32>(17424),static_cast<quint32>(18225),static_cast<quint32>(19044),static_cast<quint32>(19881),static_cast<quint32>(20736),static_cast<quint32>(21609),static_cast<quint32>(22500),static_cast<quint32>(-67643),static_cast<quint32>(18928),static_cast<quint32>(19663),static_cast<quint32>(20412),static_cast<quint32>(21175),static_cast<quint32>(21952),static_cast<quint32>(22743),static_cast<quint32>(23548),static_cast<quint32>(24367),static_cast<quint32>(25200),static_cast<quint32>(-144546),static_cast<quint32>(18451),static_cast<quint32>(19051),static_cast<quint32>(19660),static_cast<quint32>(20280),static_cast<quint32>(20908),static_cast<quint32>(21547),static_cast<quint32>(22195),static_cast<quint32>(22852),static_cast<quint32>(23520),static_cast<quint32>(24196),static_cast<quint32>(24883),static_cast<quint32>(25579),static_cast<quint32>(26284),static_cast<quint32>(27000),static_cast<quint32>(27724),static_cast<quint32>(28459),static_cast<quint32>(29203),static_cast<quint32>(29956),static_cast<quint32>(30720),static_cast<quint32>(-347438),static_cast<quint32>(18154),static_cast<quint32>(18600),static_cast<quint32>(19051),static_cast<quint32>(19507),static_cast<quint32>(19969),static_cast<quint32>(20436),static_cast<quint32>(20908),static_cast<quint32>(21386),static_cast<quint32>(21870),static_cast<quint32>(22358),static_cast<quint32>(22852),static_cast<quint32>(23352),static_cast<quint32>(23857),static_cast<quint32>(24367),static_cast<quint32>(24883),static_cast<quint32>(25404),static_cast<quint32>(25930)},
	{0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0}},

	{10,255,1,0,QT_TRANSLATE_NOOP("Char_Names","Sephiroth"),QString(":/characters/sep"),
	QStringList()<<QT_TRANSLATE_NOOP("Char_Limits","")<<QT_TRANSLATE_NOOP("Char_Limits","")<<QT_TRANSLATE_NOOP("Char_Limits","")<<QT_TRANSLATE_NOOP("Char_Limits","")<<QT_TRANSLATE_NOOP("Char_Limits","")<<QT_TRANSLATE_NOOP("Char_Limits","")<<QT_TRANSLATE_NOOP("Char_Limits",""),
	 {static_cast<quint32>(0),static_cast<quint32>(6),static_cast<quint32>(33),static_cast<quint32>(94),static_cast<quint32>(202),static_cast<quint32>(372),static_cast<quint32>(616),static_cast<quint32>(949),static_cast<quint32>(1384),static_cast<quint32>(1934),static_cast<quint32>(2614),static_cast<quint32>(3588),static_cast<quint32>(4610),static_cast<quint32>(5809),static_cast<quint32>(7200),static_cast<quint32>(8797),static_cast<quint32>(10614),static_cast<quint32>(12665),static_cast<quint32>(14965),static_cast<quint32>(17528),static_cast<quint32>(20368),static_cast<quint32>(24161),static_cast<quint32>(27694),static_cast<quint32>(31555),static_cast<quint32>(35759),static_cast<quint32>(40321),static_cast<quint32>(45255),static_cast<quint32>(50576),static_cast<quint32>(56299),static_cast<quint32>(62438),static_cast<quint32>(69008),static_cast<quint32>(77066),static_cast<quint32>(84643),static_cast<quint32>(92701),static_cast<quint32>(101255),static_cast<quint32>(110320),static_cast<quint32>(119910),static_cast<quint32>(130040),static_cast<quint32>(140725),static_cast<quint32>(151980),static_cast<quint32>(163820),static_cast<quint32>(176259),static_cast<quint32>(189312),static_cast<quint32>(202994),static_cast<quint32>(217320),static_cast<quint32>(232305),static_cast<quint32>(247963),static_cast<quint32>(264309),static_cast<quint32>(281358),static_cast<quint32>(299125),static_cast<quint32>(317625),static_cast<quint32>(336872),static_cast<quint32>(356881),static_cast<quint32>(377667),static_cast<quint32>(399245),static_cast<quint32>(421630),static_cast<quint32>(444836),static_cast<quint32>(468878),static_cast<quint32>(493771),static_cast<quint32>(519530),static_cast<quint32>(546170),static_cast<quint32>(581467),static_cast<quint32>(610297),static_cast<quint32>(640064),static_cast<quint32>(670784),static_cast<quint32>(702471),static_cast<quint32>(735141),static_cast<quint32>(768808),static_cast<quint32>(803488),static_cast<quint32>(839195),static_cast<quint32>(875945),static_cast<quint32>(913752),static_cast<quint32>(952632),static_cast<quint32>(992599),static_cast<quint32>(1033669),static_cast<quint32>(1075856),static_cast<quint32>(1119176),static_cast<quint32>(1163643),static_cast<quint32>(1209273),static_cast<quint32>(1256080),static_cast<quint32>(1304080),static_cast<quint32>(1389359),static_cast<quint32>(1441133),static_cast<quint32>(1494178),static_cast<quint32>(1548509),static_cast<quint32>(1604141),static_cast<quint32>(1661090),static_cast<quint32>(1719371),static_cast<quint32>(1778999),static_cast<quint32>(1839990),static_cast<quint32>(1902360),static_cast<quint32>(1966123),static_cast<quint32>(2031295),static_cast<quint32>(2097892),static_cast<quint32>(2165929),static_cast<quint32>(2235421),static_cast<quint32>(2306384),static_cast<quint32>(2378833),static_cast<quint32>(2452783)},
	 {static_cast<quint32>(0),static_cast<quint32>(6),static_cast<quint32>(27),static_cast<quint32>(61),static_cast<quint32>(108),static_cast<quint32>(170),static_cast<quint32>(244),static_cast<quint32>(333),static_cast<quint32>(435),static_cast<quint32>(550),static_cast<quint32>(680),static_cast<quint32>(974),static_cast<quint32>(1022),static_cast<quint32>(1199),static_cast<quint32>(1391),static_cast<quint32>(1597),static_cast<quint32>(1817),static_cast<quint32>(2051),static_cast<quint32>(2300),static_cast<quint32>(2563),static_cast<quint32>(2840),static_cast<quint32>(3793),static_cast<quint32>(3533),static_cast<quint32>(3861),static_cast<quint32>(4204),static_cast<quint32>(4562),static_cast<quint32>(4934),static_cast<quint32>(5321),static_cast<quint32>(5723),static_cast<quint32>(6139),static_cast<quint32>(6570),static_cast<quint32>(8058),static_cast<quint32>(7577),static_cast<quint32>(8058),static_cast<quint32>(8554),static_cast<quint32>(9065),static_cast<quint32>(9590),static_cast<quint32>(10130),static_cast<quint32>(10685),static_cast<quint32>(11255),static_cast<quint32>(11840),static_cast<quint32>(12439),static_cast<quint32>(13053),static_cast<quint32>(13682),static_cast<quint32>(14326),static_cast<quint32>(14985),static_cast<quint32>(15658),static_cast<quint32>(16346),static_cast<quint32>(17049),static_cast<quint32>(17767),static_cast<quint32>(18500),static_cast<quint32>(19247),static_cast<quint32>(20009),static_cast<quint32>(20786),static_cast<quint32>(21578),static_cast<quint32>(22385),static_cast<quint32>(23206),static_cast<quint32>(24042),static_cast<quint32>(24893),static_cast<quint32>(25759),static_cast<quint32>(26640),static_cast<quint32>(35297),static_cast<quint32>(28830),static_cast<quint32>(29767),static_cast<quint32>(30720),static_cast<quint32>(31687),static_cast<quint32>(32670),static_cast<quint32>(33667),static_cast<quint32>(34680),static_cast<quint32>(35707),static_cast<quint32>(36750),static_cast<quint32>(37807),static_cast<quint32>(38880),static_cast<quint32>(39967),static_cast<quint32>(41070),static_cast<quint32>(42187),static_cast<quint32>(43320),static_cast<quint32>(44467),static_cast<quint32>(45630),static_cast<quint32>(46807),static_cast<quint32>(48000),static_cast<quint32>(85279),static_cast<quint32>(51774),static_cast<quint32>(53045),static_cast<quint32>(54331),static_cast<quint32>(55632),static_cast<quint32>(56949),static_cast<quint32>(58281),static_cast<quint32>(59628),static_cast<quint32>(60991),static_cast<quint32>(62370),static_cast<quint32>(63763),static_cast<quint32>(65172),static_cast<quint32>(66597),static_cast<quint32>(68037),static_cast<quint32>(69492),static_cast<quint32>(70963),static_cast<quint32>(72449),static_cast<quint32>(73950)},
	{0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0}}
};


static const quint8 _stat_base[30][8]=
{
	{12,9,9,21,44,50,57,73},
	{13,12,11,11,17,43,53,80},
	{12,10,11,21,32,42,56,73},
	{12,13,11,15,33,40,51,69},
	{10,9,8,8,30,33,40,61},
	{12,12,14,14,23,49,55,62},
	{10,8,5,17,17,30,50,61},
	{11,10,11,16,27,33,37,58},
	{12,9,10,11,29,34,49,58},
	{9,8,8,8,26,29,42,48},
	{9,9,7,8,11,26,48,53},
	{11,10,12,17,21,49,48},
	{9,9,6,8,29,51,57,62},
	{10,9,7,16,22,43,45,54},
	{8,9,11,13,16,18,40,60},
	{9,9,11,15,23,32,48,62},
	{10,9,10,16,22,28,49,55},
	{10,10,10,13,21,39,45,57},
	{10,10,9,11,21,35,51,57},
	{9,8,9,17,32,37,42,47},
	{10,9,8,11,29,39,47,53},
	{8,7,4,20,31,36,37,46},
	{9,9,10,15,21,28,35,53},
	{9,9,8,14,18,25,44,50},
	{8,9,12,13,22,29,38,55},
	{7,7,1,8,13,20,42,46},
	{6,7,6,6,10,19,36,37},
	{6,9,8,6,7,13,31,37},
	{5,6,7,7,9,18,38,38},
	{5,6,4,9,14,20,24,30}
};
static const quint8 _stat_gradent[30][8]=
{
	{130,160,160,120,70,60,50,30},
	{120,130,133,135,120,72,55,21},
	{130,140,140,110,90,70,48,27},
	{130,140,140,110,90,79,53,32},
	{120,128,130,130,77,72,61,35},
	{120,125,117,118,93,52,44,35},
	{110,130,145,110,100,95,44,31},
	{120,135,130,110,85,70,60,35},
	{100,130,125,120,77,67,43,31},
	{110,120,122,123,80,75,55,44},
	{100,115,124,118,107,78,42,36},
	{110,120,115,102,91,37,40,40},
	{100,122,140,135,83,40,30,25},
	{110,122,130,98,83,45,44,33},
	{110,105,104,102,93,87,51,25},
	{115,127,121,108,86,68,41,24},
	{114,118,114,95,82,71,37,30},
	{112,115,111,103,83,48,39,25},
	{100,108,115,108,83,55,31,24},
	{100,111,112,87,53,45,39,34},
	{100,108,114,106,63,45,33,26},
	{100,110,127,77,50,41,40,31},
	{100,102,101,88,70,57,45,24},
	{100,100,107,85,77,60,30,24},
	{95,90,88,85,62,52,39,18},
	{80,85,115,92,78,64,27,21},
	{72,69,76,77,68,50,22,21},
	{70,53,63,70,69,58,28,20},
	{70,70,70,71,67,48,16,16},
	{65,63,76,61,49,36,28,20}
};

static const quint16 _limitbitarray[0x07] = {0,1,3,4,6,7,9};/**< Order of bytes used to flag limit breaks as learned*///Adjusted to bit offset so we can use for read/write
#endif // FF7CHAR_H
